<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KDevSwitchToBuddy" version="7" translationDomain="kdevswitchtobuddy">
<MenuBar>
  <Menu name="navigation">
    <Action name="switch_definition_declaration" group="navigation_jmp"/>
    <Action name="switch_header_source" group="navigation_jmp"/>
  </Menu>
</MenuBar>
</gui>
