/*
    SPDX-FileCopyrightText: 2010 Aleix Pol Gonzalez <aleixpol@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "vcslocationwidget.h"

KDevelop::VcsLocationWidget::VcsLocationWidget(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f)
{}
