<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kdevcodeutils" version="7" translationDomain="kdevcodeutils">
<MenuBar>
  <Menu name="code">
    <Action name="document_declaration" />
    <Action name="class_from_template" />
    <Action name="test_from_template" />
  </Menu>
</MenuBar>
</gui>

