/*
    SPDX-FileCopyrightText: 2008 David Nolden <david.nolden.kdevelop@art-master.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "icompletionsettings.h"

namespace KDevelop {
ICompletionSettings::~ICompletionSettings() {
}
}

