/*
 * This is just a test.
 * You may do with it as you please.
 */

#include "classname.h"

void ClassName::doSomething(double howMuch, bool doSomethingElse)
{
    
}

int ClassName::getSomeOtherNumber()
{
    
}
