/*
    SPDX-FileCopyrightText: 2017 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "ipagefocus.h"

namespace KDevelop
{

IPageFocus::~IPageFocus() = default;

}

