/*
    SPDX-FileCopyrightText: 2014 Denis Steckelmacher <steckdenis@yahoo.fr>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

exports.tmpdir = function () { return ""; };
exports.tmpdir();

exports.endianness = function () { return ""; };
exports.endianness();

exports.hostname = function () { return ""; };
exports.hostname();

exports.type = function () { return ""; };
exports.type();

exports.platform = function () { return ""; };
exports.platform();

exports.arch = function () { return ""; };
exports.arch();

exports.release = function () { return ""; };
exports.release();

exports.uptime = function () { return ""; };
exports.uptime();

exports.loadavg = function () { return []; };
exports.loadavg();

exports.totalmem = function () { return 1; };
exports.totalmem();

exports.freemem = function () { return 1; };
exports.freemem();

exports.cpus = function () { return []; };
exports.cpus();

exports.networkInterfaces = function () { return new Object(); };
exports.networkInterfaces();

exports.EOL = "";

