/*
    SPDX-FileCopyrightText: 2010 Benjamin Port <port.benjamin@gmail.com>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "idocumentationproviderprovider.h"

namespace KDevelop
{

IDocumentationProviderProvider::~IDocumentationProviderProvider()
{
}

}
