<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kdevsessionui" version="6" translationDomain="kdevplatform">
    <MenuBar>
        <Menu name="session">
            <Action name="new_session" />
            <Separator />
            <ActionList name="available_sessions"/>
            <Separator />
            <Action name="rename_session" />
            <Action name="delete_session" />
            <Separator />
            <Action name="quit" />
        </Menu>
    </MenuBar>
</gui>
