/*
    SPDX-FileCopyrightText: 2009 David Nolden <david.nolden.kdevelop@art-master.de>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#include "iastcontainer.h"

using namespace KDevelop;

IAstContainer::~IAstContainer()
{
}
