{% extends "cpp_qobject_pimpl_header.h" %}


{% block qobject_default_include %}
#include <QObject>
{% endblock qobject_default_include %}


{% block qobject_baseclass %}QObject{% endblock qobject_baseclass %}
