/*
    SPDX-FileCopyrightText: 2012 Miha Čančula <miha@noughmad.eu>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "itestcontroller.h"

using namespace KDevelop;

ITestController::ITestController(QObject* parent)
: QObject(parent)
{

}

ITestController::~ITestController()
{

}

