/*
    SPDX-FileCopyrightText: 2012 Miha Čančula <miha@noughmad.eu>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "itestsuite.h"

using namespace KDevelop;

ITestSuite::~ITestSuite()
{

}

