/*
    SPDX-FileCopyrightText: 2022 Milian Wolff <mail@milianw.de>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

#ifndef KDEVELOP_SANITIZER_TEST_INIT
#define KDEVELOP_SANITIZER_TEST_INIT

namespace KDevelop
{
void sanitizerTestInit(char** argv);
}

#endif // KDEVELOP_SANITIZER_TEST_INIT
