/*
 * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Samuel Weinig <sam.weinig@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

module core {

    interface [
        CustomMarkFunction,
        GenerateNativeConverter
    ] Attr : Node {

        // DOM Level 1

        readonly attribute [ConvertNullStringTo=Null] DOMString name;

        readonly attribute boolean specified;

                 attribute [ConvertNullStringTo=Null, ConvertNullToNullString] DOMString value 
                     setter raises(DOMException);

        // DOM Level 2

        readonly attribute Element ownerElement;

        // DOM Level 3

        readonly attribute boolean isId;

#if defined(LANGUAGE_OBJECTIVE_C) && LANGUAGE_OBJECTIVE_C
        // This extension is no longer needed, but it has to remain available in Objective C, as it's public API.
        readonly attribute CSSStyleDeclaration style;
#endif
    };

}
