 /*
    SPDX-FileCopyrightText: 2009 Andreas Pakulat <apaku@gmx.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "launchconfigurationpage.h"

namespace KDevelop
{

LaunchConfigurationPage::LaunchConfigurationPage( QWidget* parent ) : QWidget( parent ) 
{
}

}

